﻿using System;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using Microsoft.IdentityModel.Configuration;
using Microsoft.IdentityModel.Protocols.WSTrust;
using Microsoft.IdentityModel.SecurityTokenService;

namespace BasicSTS
{
    class Program
    {
        static void Main(string[] args)
        {
            string stsAddress = "http://localhost:9000/STS";

            // load cert with private key.
            X509Certificate2 stsSignCert = new X509Certificate2("localhost.pfx","a");

            SecurityTokenServiceConfiguration stsConfig =
                new SecurityTokenServiceConfiguration("http://localsts.net", new X509SigningCredentials(stsSignCert));
            stsConfig.DefaultTokenLifetime = TimeSpan.FromSeconds(10);
            stsConfig.SecurityTokenService = typeof(CustomSecurityTokenService);

            // host STS as WS-Trust endpoint.
            WSTrustServiceHost stsHost = new WSTrustServiceHost(new MyWSTrustContract(stsConfig), new Uri(stsAddress));
            var bid = new WSHttpBinding("AnnonyForCertificate");

            stsHost.AddServiceEndpoint(typeof(IWSTrustFeb2005SyncContract),
               bid, new Uri(stsAddress));

            stsHost.Credentials.ServiceCertificate.Certificate = stsSignCert;

            stsHost.Open();
           
            Console.WriteLine("Local STS is ready ...");

            foreach(var ep in stsHost.Description.Endpoints)
                Console.WriteLine(ep.Address.Uri.AbsoluteUri);
            Console.ReadLine();

            stsHost.Close();

        }
    }
}
