﻿using System;
using System.IdentityModel.Tokens;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.Threading;
using EmployeeServiceContract;
using Microsoft.IdentityModel.Claims;
using Microsoft.IdentityModel.Configuration;
using Microsoft.IdentityModel.Tokens;

namespace EmployeeService
{
    public class EmployeeService : IEmployeeService
    {
        #region IEmployeeService Members

        public bool ChangePassword(string oldPwd, string newPwd)
        {
            Console.WriteLine("Primary Identity= \"{0}\", authentication type = \"{1}\"", 
                OperationContext.Current.ServiceSecurityContext.PrimaryIdentity.Name,
                OperationContext.Current.ServiceSecurityContext.PrimaryIdentity.AuthenticationType
                );

            DumpClaims();
            
            Console.WriteLine("Password changed...");
            
            return true;
        }
       #endregion

        private void DumpClaims()
        {
            var css = OperationContext.Current.ServiceSecurityContext.AuthorizationContext.ClaimSets;
            foreach (var cs in css)
            {
                Console.WriteLine("***** Claimset *******");
                foreach (var c in cs)
                {
                    Console.WriteLine(c.ClaimType + " : " + c.Resource);
                }
            }
        }
    }

    public class TestIssuerRegistry : IssuerNameRegistry
    {
        public override string GetIssuerName(SecurityToken securityToken)
        {
            return "unkown";
        }
    }


    class Program
    {
        static void Main(string[] args)
        {
            var sh = new ServiceHost(typeof(EmployeeService));

            //zamd.local.pfx: private key cert. used to identify the service and decrypt the token issued by Geneva
            SetupServiceCert(sh, @"zamd.local.pfx");

            ServiceConfiguration config = new ServiceConfiguration
            {
                IssuerNameRegistry = new TestIssuerRegistry()
            };
            FederatedServiceCredentials.ConfigureServiceHost(sh,config);

            sh.Open();
            
            foreach (var ep in sh.Description.Endpoints)
            {
                Console.WriteLine(ep.ListenUri);
            }

            Console.ReadLine();
            sh.Close();
        }

        private static void SetupServiceCert(ServiceHost sh,string pfxPath)
        {
            var cert = new X509Certificate2(pfxPath, "a"); 
            sh.Credentials.ServiceCertificate.Certificate = cert;

            //sh.Credentials.IssuedTokenAuthentication.KnownCertificates.Add(cert);

            //sh.Credentials.IssuedTokenAuthentication.CertificateValidationMode = 
            //    System.ServiceModel.Security.X509CertificateValidationMode.None;
        }
    }
}
