﻿using System;
using System.IdentityModel.Tokens;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.ServiceModel.Security;
using Microsoft.IdentityModel.Protocols.WSTrust;

namespace SecurityExtensions
{
    public static class WSTrustClientExtension
    {
        static void EnsureOk(WSTrustClient client)
        {
            if (client.State == CommunicationState.Opened)
                throw new InvalidOperationException("This operation is not allowed while the object is in Opened state.");
        }
        public static WSTrustClient SetActingAs(this WSTrustClient client, SecurityToken actAs)
        {
            EnsureOk(client);
            return WSTrustClientEx.Clone(client, null, null, actAs);
        }
        public static WSTrustClient SetOnBehalfOf(this WSTrustClient client, SecurityToken onBehalfOf)
        {
            EnsureOk(client);
            return WSTrustClientEx.Clone(client, null, onBehalfOf, null);
        }

        public static WSTrustClient SetIssuedToken(this WSTrustClient client, SecurityToken issuedToken)
        {
            EnsureOk(client);
            return WSTrustClientEx.Clone(client, issuedToken, null, null);
        }

        private class WSTrustClientEx : WSTrustClient
        {
            private WSTrustClientEx(Binding binding, EndpointAddress epa, TrustVersion trustVersion, ClientCredentials clientCredentials)
                : base(binding, epa, trustVersion, clientCredentials)
            {

            }
            static SecurityToken onBehalfOf;
            static SecurityToken actAs;
            static SecurityToken issuedToken;
            internal static WSTrustClient Clone(WSTrustClient client, SecurityToken issuedToken, SecurityToken onBehalfOf, SecurityToken actAs)
            {
                var clone = new WSTrustClientEx(client.Endpoint.Binding, client.Endpoint.Address, client.TrustVersion, client.ClientCredentials);
                clone.WSTrustRequestSerializer = client.WSTrustRequestSerializer;
                clone.WSTrustResponseSerializer = client.WSTrustResponseSerializer;

                WSTrustClientEx.onBehalfOf = onBehalfOf;
                WSTrustClientEx.actAs = actAs;
                WSTrustClientEx.issuedToken = issuedToken;
                return clone;
            }

            protected override IWSTrustContract CreateChannel()
            {
                if (onBehalfOf != null)
                {
                    this.ChannelFactory.ConfigureChannelFactory();
                    return this.ChannelFactory.CreateChannelOnBehalfOf(onBehalfOf);
                }
                if (actAs != null)
                {
                    this.ChannelFactory.ConfigureChannelFactory();
                    return this.ChannelFactory.CreateChannelActingAs(actAs);
                }
                if (issuedToken != null)
                {
                    this.ChannelFactory.ConfigureChannelFactory();
                    return this.ChannelFactory.CreateChannelWithIssuedToken(issuedToken);
                }

                return base.CreateChannel();
            }
        }

    }
}