﻿using System;
using System.IdentityModel.Selectors;
using System.IdentityModel.Tokens;
using System.IO;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.ServiceModel.Security;
using System.Xml;
using Microsoft.IdentityModel.Protocols.WSTrust;
using Microsoft.IdentityModel.SecurityTokenService;
using Microsoft.IdentityModel.Tokens;
using Microsoft.IdentityModel.Tokens.Saml11;


public static class Util
{
    public static string ExtractSAMLAssertion(SecurityToken genericXmlToken, X509Certificate2 rpCert)
    {
        var token = genericXmlToken as GenericXmlSecurityToken;
        var han = new EncryptedSecurityTokenHandler();

        han.ContainingCollection = new SecurityTokenHandlerCollection() { new Saml11SecurityTokenHandler() };
        han.Configuration.ServiceTokenResolver = new EncryptedKeyResolver(rpCert);

        var sr = new StringReader(token.TokenXml.OuterXml);
        var rdr2 = XmlReader.Create(sr);

        var rtok = han.ReadToken(rdr2) as SamlSecurityToken;

        var ms = new MemoryStream();
        var memWriter = XmlDictionaryWriter.CreateTextWriter(ms);
        memWriter.WriteStartElement("testRoot");
        foreach (var stat in rtok.Assertion.Statements)
        {
            stat.WriteXml(memWriter, new SamlSerializer(), new WSSecurityTokenSerializer());
        }
        memWriter.WriteEndElement();
        
        memWriter.Flush();
        ms.Seek(0, SeekOrigin.Begin);

        var txtStat = new StreamReader(ms).ReadToEnd();
        return txtStat;
    }

    public static SecurityToken GetTokenFromLocalSTS(string appliesTo)
    {
        var localSTSBinding = new WSHttpBinding("AnnonyForCertificate");

        //only public key cert. use to secure communication.
        var localSTSCert = new X509Certificate2(@"MyCustomSTSPublicKey.cer");
        var localSTSIdentity = new X509CertificateEndpointIdentity(localSTSCert);
        var localSTSAddress = new EndpointAddress(new Uri("http://localhost:9000/STS"), localSTSIdentity);

        WSTrustClient client = new WSTrustClient(localSTSBinding, localSTSAddress, TrustVersion.WSTrustFeb2005, new ClientCredentials());
        RequestSecurityToken rst = new RequestSecurityToken(RequestTypeConstants.Issue);
        rst.AppliesTo = new EndpointAddress(appliesTo);


        RequestSecurityTokenResponse rstr;
        var token = client.Issue(rst, out rstr);
        client.Close();
        return token;
    }

    public static SecurityToken GetTokenFromLocalSTS(string appliesTo, Binding binding)
    {
        var localSTSBinding = binding;

        //only public key cert. use to secure communication.
        var localSTSCert = new X509Certificate2(@"MyCustomSTSPublicKey.cer");
        var localSTSIdentity = new X509CertificateEndpointIdentity(localSTSCert);
        var localSTSAddress = new EndpointAddress(new Uri("http://localhost:9000/STS"), localSTSIdentity);

        WSTrustClient client = new WSTrustClient(localSTSBinding, localSTSAddress, TrustVersion.WSTrustFeb2005, new ClientCredentials());
        RequestSecurityToken rst = new RequestSecurityToken(RequestTypeConstants.Issue);
        rst.AppliesTo = new EndpointAddress(appliesTo);


        RequestSecurityTokenResponse rstr;
        var token = client.Issue(rst, out rstr);
        client.Close();
        return token;
    }

}

public class EncryptedKeyResolver : SecurityTokenResolver
{
    public EncryptedKeyResolver()
    {
    }
    // Relying party certificate - must have a private key.
    X509Certificate2 rpCert;
    public EncryptedKeyResolver(X509Certificate2 rpCert)
    {
        this.rpCert = rpCert;

    }
    protected override bool TryResolveSecurityKeyCore(SecurityKeyIdentifierClause keyIdentifierClause, out SecurityKey key)
    {
        if (!rpCert.HasPrivateKey)
            throw new Exception("No Private Key found in the certificate..");
        key = null;
        try
        {
            var ekec = keyIdentifierClause as EncryptedKeyIdentifierClause;
            
            if (ekec != null)
            {
                switch (ekec.EncryptionMethod)
                {
                    case "http://www.w3.org/2001/04/xmlenc#rsa-oaep-mgf1p":
                        {
                            var encKey = ekec.GetEncryptedKey();

                            var rsa = rpCert.PrivateKey as RSACryptoServiceProvider;
                            var decKey = rsa.Decrypt(encKey, true);

                            key = new InMemorySymmetricSecurityKey(decKey);
                            return true;
                        }
                }
                var data = ekec.GetEncryptedKey();
                var id = ekec.EncryptingKeyIdentifier;
            }
        }
        catch (Exception eexx)
        {
            Console.WriteLine(eexx);
            throw eexx;
        }
        return false;
    }

    protected override bool TryResolveTokenCore(SecurityKeyIdentifierClause keyIdentifierClause, out SecurityToken token)
    {
        throw new NotImplementedException();
    }

    protected override bool TryResolveTokenCore(SecurityKeyIdentifier keyIdentifier, out SecurityToken token)
    {
        throw new NotImplementedException();
    }
}