﻿using System;
using System.IdentityModel.Tokens;
using System.Net;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.ServiceModel.Security;
using System.ServiceModel.Security.Tokens;
using Microsoft.IdentityModel.Protocols.WSTrust;
using Microsoft.IdentityModel.Protocols.WSTrust.Bindings;
using Microsoft.IdentityModel.SecurityTokenService;

namespace WCFClient
{
    class Program
    {
        static void Main(string[] args)
        {
            ServicePointManager.ServerCertificateValidationCallback =
                delegate(object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors sslPolicyErrors)
                {
                    Console.WriteLine("Validating SSL certificate:  {0}", certificate.Subject);
                    //For testing purposes only, treate all certificates valid
                    return true;
                };


            WIFExplicitTokenAcquire();

            WCFExplicitTokenAcquire();

            // Start EmployeeService before running this
            WCFImplicitTokenAcquire();

            // Start local STS before running this.
            WIFExplicitFederation();
        }

        private static void WCFExplicitTokenAcquire()
        {
            string baseUri = "https://bccoss.com/";

            //Limited edition of WSTrustClient:)
            IssuedSecurityTokenProvider provider = new IssuedSecurityTokenProvider();
            provider.SecurityTokenSerializer = new WSSecurityTokenSerializer();

            //Relying Party's identifier
            provider.TargetAddress = new EndpointAddress(new Uri("http://local.zamd.net"));
            provider.IssuerAddress = new EndpointAddress(new Uri(baseUri + "Trust/2005/WindowsTransport/"));

            provider.SecurityAlgorithmSuite = SecurityAlgorithmSuite.Basic256;
            provider.MessageSecurityVersion = MessageSecurityVersion.WSSecurity10WSTrustFebruary2005WSSecureConversationFebruary2005WSSecurityPolicy11BasicSecurityProfile10;

            HttpsTransportBindingElement tbe = new HttpsTransportBindingElement();
            tbe.AuthenticationScheme = AuthenticationSchemes.Negotiate;
            CustomBinding stsBinding = new CustomBinding(tbe);

            provider.IssuerBinding = stsBinding;
            provider.Open();
            //Request a token from ADFS STS
            SecurityToken issuedToken = provider.GetToken(TimeSpan.FromSeconds(30));

            X509Certificate2 rpCert = new X509Certificate2("zamd.local.pfx", "a");
            string xml = Util.ExtractSAMLAssertion(issuedToken, rpCert);
            Console.WriteLine(xml);
        }

        private static void WIFExplicitTokenAcquire()
        {
            string baseUri = "https://bccoss.com/";

            WindowsWSTrustBinding binding = new WindowsWSTrustBinding(SecurityMode.Transport);

            WSTrustClient tokenClient = new WSTrustClient(binding,
                new EndpointAddress(baseUri + "Trust/2005/WindowsTransport/"),
                TrustVersion.WSTrustFeb2005,
                new ClientCredentials());

            //create a token issuance issuance
            RequestSecurityToken rst =
                new RequestSecurityToken(WSTrustFeb2005Constants.RequestTypes.Issue);
            //Relying Party’s identifier
            rst.AppliesTo = new EndpointAddress("http://local.zamd.net/");
            //call ADFS STS
            SecurityToken token = tokenClient.Issue(rst);

            X509Certificate2 rpCert = new X509Certificate2("zamd.local.pfx", "a");
            string xml = Util.ExtractSAMLAssertion(token, rpCert);
            Console.WriteLine(xml);
        }

        private static void WCFImplicitTokenAcquire()
        {
            ServiceReference1.EmployeeServiceClient proxy = new WCFClient.ServiceReference1.EmployeeServiceClient();
            proxy.ChangePassword("test", "newTest");
            Console.WriteLine("done.");
        }

        // Federate ADFS 2.0 with a custom STS and explicity acquires a raw token using WSTrustClient.
        private static void WIFExplicitFederation()
        {
            string adfsStsUri = "http://bccoss.com/Trust/2005/IssuedTokenAsymmetricBasic256";

            //binding for local STS(IP-STS)
            WSHttpBinding localStsBinding = new WSHttpBinding(SecurityMode.Message);

            localStsBinding.Security.Message.ClientCredentialType = MessageCredentialType.None;
            localStsBinding.Security.Message.EstablishSecurityContext = false;
            localStsBinding.Security.Message.NegotiateServiceCredential = false;

            EndpointAddress localStsEpr = new EndpointAddress(
                new Uri("http://localhost:9000/STS/"),
                new X509CertificateEndpointIdentity(new X509Certificate2(@"MyCustomSTSPublicKey.cer")));

            //This binding will transparently acquire all the intermediate tokens as part of the call. (R-STS)
            IssuedTokenWSTrustBinding fedBinding = new IssuedTokenWSTrustBinding(localStsBinding, localStsEpr);
            fedBinding.TrustVersion = TrustVersion.WSTrustFeb2005;


            EndpointAddress adfsStsEpr = new EndpointAddress(
                new Uri(adfsStsUri),
                new X509CertificateEndpointIdentity(new X509Certificate2("AdfsStsPubicKeyOnly.cer")));

            WSTrustClient trustClient = new WSTrustClient(fedBinding, adfsStsEpr, TrustVersion.WSTrustFeb2005, null);

            //Create a security token request
            RequestSecurityToken rst = new RequestSecurityToken(RequestTypeConstants.Issue);
            //Set Relying Party's identifier accordingly
            rst.AppliesTo = new EndpointAddress("http://local.zamd.net");

            SecurityToken finalToken = trustClient.Issue(rst);
            X509Certificate2 rpCert = new X509Certificate2("zamd.local.pfx", "a");
            string xml = Util.ExtractSAMLAssertion(finalToken, rpCert);
            Console.WriteLine(xml);

        }
    }
}
