﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Web;
using Microsoft.IdentityModel.Claims;
using Microsoft.IdentityModel.Protocols.WSTrust;
using Microsoft.IdentityModel.SecurityTokenService;

namespace passiveSTS
{
    public class SimpleScopeProvider
    {
        static List<FederatedPartner> registeredPartners;

        static SimpleScopeProvider()
        {
            registeredPartners = new List<FederatedPartner>{ 
                new FederatedPartner("http://www.mainportal.local"),
                new FederatedPartner("http://www.aws.local")
            };
            registeredPartners.ForEach(r => r.Init());
        }
        public static Scope GetScope(RequestSecurityToken rst)
        {
            var partner = GetFederationPartner(rst.AppliesTo.Uri);
            if (partner == null)
                    throw new Exception("Invalid partner url.");

            var appliesToAddress = rst.AppliesTo.Uri.AbsoluteUri;
            Scope targetScope = new Scope(appliesToAddress);
            targetScope.EncryptingCredentials = new X509EncryptingCredentials(partner.Certificate);
            targetScope.Properties["OutputClaimsIdentity"] = new ClaimsIdentity(partner.OutputClaims);
            return targetScope;
        }

        private static FederatedPartner GetFederationPartner(Uri appliesTo)
        {
            return registeredPartners.FirstOrDefault(u => u.Address.IsBaseOf(appliesTo));
        }



    }

    public static class Extensions
    {
        public static void Init(this FederatedPartner source)
        {
            var certPath = HttpContext.Current.Server.MapPath("~/certificate-store/");
            if (source.Address.Equals("http://www.aws.local")) //another web site
            {
                source.Certificate = new X509Certificate2(certPath + "aws.cer");
                source.OutputClaims.Add(new Claim("urn:passiveSts.com/authenticatedFor", "aws web application"));
            }
            else if (source.Address.Equals("http://www.mainportal.local"))
            {
                source.Certificate = new X509Certificate2(certPath + "mainportal.cer");
                source.OutputClaims.Add(new Claim("urn:passiveSts.com/authenticatedFor", "passiveSts portal"));
            }
            // add shared claims...
            source.OutputClaims.Add(new Claim(System.IdentityModel.Claims.ClaimTypes.Name, HttpContext.Current.User.Identity.Name));
            source.OutputClaims.Add(new Claim("urn:passiveSts.com/authenticatedBy", "passiveSts identity STS"));
        }
    }

    public class FederatedPartner
    {
        public FederatedPartner(string address)
            : this(new Uri(address))
        { }
        public FederatedPartner(Uri address)
        {
            this.Address = address;
            this.OutputClaims = new List<Claim>();
        }
        public Uri Address { get; private set; }
        public X509Certificate2 Certificate { get; set; }
        public List<Claim> OutputClaims { get; set; }
    }
}
